extern crate sdl2;

use sdl2::*;
use sdl2::render::TextureCreator;
use sdl2::video::Window;
use sdl2::render::Canvas;
use sdl2::render::Texture;
use sdl2::pixels::PixelFormatEnum;
use sdl2::event::Event;
use sdl2::keyboard::Keycode;
use sdl2::rect::Rect;
use sdl2::video::WindowContext;

//
// screen abstraction wrapper
// 
struct RRScreen<'a> {
    canvas: Canvas<Window>,
    tex_cr: TextureCreator<WindowContext>,
    tex: Texture<'a>,
    event_pump: EventPump
}

impl RRScreen<'_> {
    fn new(title: &str, w:u32, h:u32) -> RRScreen {
        let cx = sdl2::init().unwrap();
        let wn = cx.video().unwrap().window(title, w, h).opengl().build().unwrap();
        let c = wn.into_canvas().build().unwrap();
        RRScreen {
            canvas: c,
            tex_cr: c.texture_creator(),
            tex: tex_cr.create_texture_streaming(PixelFormatEnum::ARGB32, w, h).unwrap(),
            event_pump: cx.event_pump().unwrap()
        }
    }
}

//
// XXX
//

pub fn main() -> Result<(), String> {
    let _sc = RRScreen::new("Hello World", 800, 600);

// XXX play code below... for reference
    let sdl_context = sdl2::init()?;
    let video_subsystem = sdl_context.video()?;

    let window = video_subsystem
        .window("rust-sdl2 demo: Video", 800, 600)
        .position_centered()
        .opengl()
        .build()
        .map_err(|e| e.to_string())?;

    let mut canvas = window.into_canvas().build().map_err(|e| e.to_string())?;
    let texture_creator = canvas.texture_creator();

    let mut texture = texture_creator
        .create_texture_streaming(PixelFormatEnum::ARGB32, 800, 600)
        .map_err(|e| e.to_string())?;
    let mut event_pump = sdl_context.event_pump()?;

    'running: loop {
        // Create a red-green gradient
        texture.with_lock(None, |buffer: &mut [u8], pitch: usize| {
            for y in 0..600 {
                let mut seed: u8 = rand::random();
                for x in 0..800 {
                    let offset = y * pitch + x * 4;
                    buffer[offset] = 0xff;
                    buffer[offset + 1] = (x as u8) ^ seed;
                    buffer[offset + 1] = (y as u8) ^ seed;
                    buffer[offset + 2] = seed as u8;
                    if seed == 255 {
                        seed = 0;
                    }
                    else {
                        seed = seed + 1;
                    }
                }
            }
        })?;

        canvas.clear();
        canvas.copy(&texture, None, Some(Rect::new(0, 0, 800, 600)))?;
        canvas.present();

        for event in event_pump.poll_iter() {
            match event {
                Event::Quit { .. }
                | Event::KeyDown {
                    keycode: Some(Keycode::Escape),
                    ..
                } => break 'running,
                _ => {}
            }
        }
        // The rest of the game loop goes here...
    }

    Ok(())
}